#include <toast_preferences.h>
#include <toast_timer.h>
#include <resource_ids.h>

DmOpenRef
ToastGetPreferences()
{
  DmOpenRef pref_ref;
  Err error = 0;
  DmSearchStateType search;
  UInt16 card;
  LocalID dbid;
  error = DmGetNextDatabaseByTypeCreator (true, &search, sysResTAppPrefs,
					  ToastTimerCreator, false, &card,
					  &dbid);
  if (error != errNone) {
    if (error != dmErrCantFind) {
      ErrDisplay("Failed to preferences DB");
      return 0;
    }

    error = DmCreateDatabase(0,ToastTimerPrefsName,ToastTimerCreator,
			     sysResTAppPrefs, true);
    if (error) {
      ErrDisplay("Failed to create preferences DB");
      return 0;
    }
    error = DmGetNextDatabaseByTypeCreator (true, &search, sysResTAppPrefs,
					    ToastTimerCreator, false, &card,
					    &dbid);
    if (error != errNone) {
      ErrDisplay("Failed to find new preferences DB");
      return 0;
    }
    pref_ref = DmOpenDatabase(card,dbid, dmModeReadWrite);
    if (pref_ref == 0) {
      DmDeleteDatabase(0,dbid);
      ErrDisplay("Failed to open new preferences DB");
      return 0;
    }
  } else {
    pref_ref = DmOpenDatabase(card,dbid, dmModeReadWrite);
    if (pref_ref == 0) {
      ErrDisplay("Failed to open preferences DB");
      return 0;
    }
  }

  return pref_ref;
}

void
ToastReleasePreferences(DmOpenRef pref_ref)
{
  DmCloseDatabase(pref_ref);
}


static Int32
GetIntPref(DmResID res)
{
  Int32 v;
  MemHandle res_hndl = DmGetResource('tint',res); 
  if (res_hndl == NULL) return -1;
  v = *(Int32*)MemHandleLock(res_hndl);
  MemHandleUnlock(res_hndl);
  return v;
}

static void
SetIntPref(DmOpenRef ref,DmResID res_id, Int32 v)
{
  UInt16 res_index;
  MemHandle res_hndl;
  MemPtr res_ptr;
  res_index = DmFindResource (ref, 'tint', res_id, NULL);
  if (res_index == -1) {
    res_hndl = DmNewResource(ref, 'tint', res_id, sizeof(Int32));
    if (!res_hndl) {
      ErrDisplay("Failed to create new preference record");
      return;
    }
    
  } else {
    res_hndl = DmGetResourceIndex(ref, res_index);
    if (!res_hndl) {
      ErrDisplay("Failed to get preference record");
      return;
    }
  }

  res_ptr = MemHandleLock(res_hndl);
  DmWrite(res_ptr,0,&v, sizeof(Int32));
  MemHandleUnlock(res_hndl);
  DmReleaseResource(res_hndl);
}

Int32
ToastGetPrefRepeats()
{
  return GetIntPref(PREFS_REPEATS);
}

void
ToastSetPrefRepeats(DmOpenRef ref, Int32 v)
{
  SetIntPref(ref,PREFS_REPEATS, v);
}

Int32
ToastGetPrefRepeatInterval()
{
 return GetIntPref(PREFS_REPEAT_INTERVAL); 
}

void
ToastSetPrefRepeatInterval(DmOpenRef ref, Int32 v)
{
  SetIntPref(ref,PREFS_REPEAT_INTERVAL, v);
}

Boolean
ToastGetPrefUseSysMIDIOnly()
{
 return GetIntPref(PREFS_SYS_MIDI_ONLY) != 0; 
}

void
ToastSetPrefUseSysMIDIOnly(DmOpenRef ref, Boolean v)
{
  SetIntPref(ref,PREFS_SYS_MIDI_ONLY, v ? 1 : 0);
}
